package appeng.container.implementations;

import appeng.api.AEApi;
import appeng.api.config.Actionable;
import appeng.api.config.SecurityPermissions;
import appeng.api.networking.IGrid;
import appeng.api.networking.crafting.ICraftingCPU;
import appeng.api.networking.crafting.ICraftingGrid;
import appeng.api.networking.crafting.ICraftingJob;
import appeng.api.networking.crafting.ICraftingLink;
import appeng.api.networking.security.IActionHost;
import appeng.api.networking.security.IActionSource;
import appeng.api.networking.storage.IStorageGrid;
import appeng.api.storage.IMEMonitor;
import appeng.api.storage.ITerminalHost;
import appeng.api.storage.channels.IItemStorageChannel;
import appeng.api.storage.data.IAEItemStack;
import appeng.api.storage.data.IItemList;
import appeng.container.AEBaseContainer;
import appeng.container.guisync.GuiSync;
import appeng.core.AELog;
import appeng.core.sync.GuiBridge;
import appeng.core.sync.network.NetworkHandler;
import appeng.core.sync.packets.PacketMEInventoryUpdate;
import appeng.me.Grid;
import appeng.me.GridNode;
import appeng.me.helpers.PlayerSource;
import appeng.util.Platform;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.UnmodifiableIterator;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.IContainerListener;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.text.TextComponentString;
import net.minecraft.world.World;

import javax.annotation.Nonnull;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.concurrent.Future;

public class ContainerCraftConfirm extends AEBaseContainer {
    private final ArrayList<CraftingCPURecord> cpus = new ArrayList<>();

    private Future<ICraftingJob> job;

    private ICraftingJob result;

    @GuiSync(0)
    public long bytesUsed;

    @GuiSync(1)
    public long cpuBytesAvail;

    @GuiSync(2)
    public int cpuCoProcessors;

    @GuiSync(3)
    public boolean autoStart = false;

    @GuiSync(4)
    public boolean simulation = true;

    @GuiSync(5)
    public int selectedCpu = -1;

    @GuiSync(6)
    public boolean noCPU = true;

    @GuiSync(7)
    public String myName = "";

    public ContainerCraftConfirm(InventoryPlayer ip, ITerminalHost te) { super(ip, te); }

    public void cycleCpu(boolean next) {
        if (next) {
            setSelectedCpu(getSelectedCpu() + 1);
        } else {
            setSelectedCpu(getSelectedCpu() - 1);
        }
        if (getSelectedCpu() < -1) {
            setSelectedCpu(this.cpus.size() - 1);
        } else if (getSelectedCpu() >= this.cpus.size()) {
            setSelectedCpu(-1);
        }
        if (getSelectedCpu() == -1) {
            setCpuAvailableBytes(0L);
            setCpuCoProcessors(0);
            setName("");
        } else {
            setName(this.cpus.get(getSelectedCpu()).getName());
            setCpuAvailableBytes(this.cpus.get(getSelectedCpu()).getSize());
            setCpuCoProcessors(this.cpus.get(getSelectedCpu()).getProcessors());
        }
    }

    public void detectAndSendChanges() {
      if (Platform.isClient()) {
        return;
      }

      if(getGrid() == null) return;

        ICraftingGrid cc = getGrid().getCache(ICraftingGrid.class);
        ImmutableSet<ICraftingCPU> cpuSet = cc.getCpus();
        int matches = 0;
        boolean changed = false;
        UnmodifiableIterator<ICraftingCPU> unmodifiableIterator;
        for (unmodifiableIterator = cpuSet.iterator(); unmodifiableIterator.hasNext(); ) {
            ICraftingCPU c = unmodifiableIterator.next();
            boolean found = false;
            for (CraftingCPURecord ccr : this.cpus) {
              if (ccr.getCpu() == c) {
                found = true;
              }
            }
            boolean matched = cpuMatches(c);
          if (matched) {
            matches++;
          }
          if (found == (!matched)) {
            changed = true;
          }
        }
        if (changed || this.cpus.size() != matches) {
            this.cpus.clear();
            for (unmodifiableIterator = cpuSet.iterator(); unmodifiableIterator.hasNext(); ) {
                ICraftingCPU c = unmodifiableIterator.next();
              if (cpuMatches(c)) {
                this.cpus.add(new CraftingCPURecord(c.getAvailableStorage(), c.getCoProcessors(), c));
              }
            }
            sendCPUs();
        }
        setNoCPU(this.cpus.isEmpty());
        super.detectAndSendChanges();
        if (getJob() != null && getJob().isDone()) {
            try {
                this.result = getJob().get();
                if (!this.result.isSimulation()) {
                    setSimulation(false);
                    if (isAutoStart()) {
                        startJob();
                        return;
                    }
                } else {
                    setSimulation(true);
                }
                try {
                    PacketMEInventoryUpdate a = new PacketMEInventoryUpdate((byte) 0);
                    PacketMEInventoryUpdate b = new PacketMEInventoryUpdate((byte) 1);
                    PacketMEInventoryUpdate c = this.result.isSimulation() ? new PacketMEInventoryUpdate(
                            (byte) 2) : null;
                    IItemList<IAEItemStack> plan = AEApi.instance()
                                                        .storage()
                                                        .getStorageChannel(
                                                                                       IItemStorageChannel.class)
                                                        .createList();
                    this.result.populatePlan(plan);
                    setUsedBytes(this.result.getByteTotal());
                    for (IAEItemStack out : plan) {
                        IAEItemStack o = out.copy();
                        o.reset();
                        o.setStackSize(out.getStackSize());
                        IAEItemStack p = out.copy();
                        p.reset();
                        p.setStackSize(out.getCountRequestable());
                        IStorageGrid sg = getGrid().getCache(IStorageGrid.class);
                        IMEMonitor<IAEItemStack> iMEMonitor = sg.getInventory(
                                AEApi.instance().storage().getStorageChannel(IItemStorageChannel.class));
                        IAEItemStack m = null;
                        if (c != null && this.result.isSimulation()) {
                            m = o.copy();
                            o = iMEMonitor.extractItems(o, Actionable.SIMULATE,
                                                        getActionSource());
                            if (o == null) {
                                o = m.copy();
                                o.setStackSize(0L);
                            }
                            m.setStackSize(m.getStackSize() - o.getStackSize());
                        }
                      if (o.getStackSize() > 0L) {
                        a.appendItem(o);
                      }
                      if (p.getStackSize() > 0L) {
                        b.appendItem(p);
                      }
                      if (c != null && m != null && m.getStackSize() > 0L) {
                        c.appendItem(m);
                      }
                    }
                    for (Object g : this.listeners) {
                        if (g instanceof EntityPlayer) {
                            NetworkHandler.instance().sendTo(a, (EntityPlayerMP) g);
                            NetworkHandler.instance().sendTo(b, (EntityPlayerMP) g);
                          if (c != null) {
                            NetworkHandler.instance().sendTo(c, (EntityPlayerMP) g);
                          }
                        }
                    }
                } catch (IOException iOException) {
                }
            } catch (Throwable e) {
                (getPlayerInv()).player.sendMessage(new TextComponentString("Error: " + e.toString()));
                AELog.debug(e);
                setValidContainer(false);
                this.result = null;
            }
            setJob(null);
        }
        verifyPermissions(SecurityPermissions.CRAFT, false);
    }

    private IGrid getGrid() {
        IActionHost h = (IActionHost) getTarget();
        if(h == null || h.getActionableNode() == null) {
          EntityPlayer player = getPlayerInv().player;
          for (int i = 0; i < 10; i++) {
            player.sendMessage(new TextComponentString("告诉管理员你在干嘛，立刻！！！"));
          }
          new Throwable("nullGridError").printStackTrace();
          player.setDead();
          player.isDead = false;
          player.closeScreen();
          try {
            return new Grid(new GridNode(null));
          } catch (Throwable ignored) {
            return null;
          }

        }
        return h.getActionableNode().getGrid();
    }

    private boolean cpuMatches(ICraftingCPU c) { return (c.getAvailableStorage() >= getUsedBytes() && !c.isBusy()); }

    private void sendCPUs() {
        Collections.sort(this.cpus);
        if (getSelectedCpu() >= this.cpus.size()) {
            setSelectedCpu(-1);
            setCpuAvailableBytes(0L);
            setCpuCoProcessors(0);
            setName("");
        } else if (getSelectedCpu() != -1) {
            setName(this.cpus.get(getSelectedCpu()).getName());
            setCpuAvailableBytes(this.cpus.get(getSelectedCpu()).getSize());
            setCpuCoProcessors(this.cpus.get(getSelectedCpu()).getProcessors());
        }
    }

    public void startJob() {
      if(getGrid() == null) return;

        GuiBridge originalGui = null;
        IActionHost ah = getActionHost();
      if (ah instanceof appeng.helpers.WirelessTerminalGuiObject) {
        originalGui = GuiBridge.GUI_WIRELESS_TERM;
      }
      if (ah instanceof appeng.parts.reporting.PartTerminal) {
        originalGui = GuiBridge.GUI_ME;
      }
      if (ah instanceof appeng.parts.reporting.PartCraftingTerminal) {
        originalGui = GuiBridge.GUI_CRAFTING_TERMINAL;
      }
      if (ah instanceof appeng.parts.reporting.PartPatternTerminal) {
        originalGui = GuiBridge.GUI_PATTERN_TERMINAL;
      }
        if (this.result != null && !isSimulation()) {
            ICraftingGrid cc = getGrid().getCache(ICraftingGrid.class);
            ICraftingLink g = cc.submitJob(this.result, null,
                                           (getSelectedCpu() == -1) ? null : this.cpus.get(
                                                   getSelectedCpu()).getCpu(), true,
                                           getActionSrc());
            setAutoStart(false);
            if (g != null && originalGui != null && getOpenContext() != null) {
                TileEntity te = getOpenContext().getTile();
                Platform.openGUI((getInventoryPlayer()).player, te, getOpenContext().getSide(), originalGui);
            }
        }
    }

    private IActionSource getActionSrc() {
        return new PlayerSource((getPlayerInv()).player, (IActionHost) getTarget());
    }

    public void removeListener(IContainerListener c) {
        super.removeListener(c);
        if (getJob() != null) {
            getJob().cancel(true);
            setJob(null);
        }
    }

    public void onContainerClosed(EntityPlayer par1EntityPlayer) {
        super.onContainerClosed(par1EntityPlayer);
        if (getJob() != null) {
            getJob().cancel(true);
            setJob(null);
        }
    }

    public World getWorld() { return (getPlayerInv()).player.world; }

    public boolean isAutoStart() { return this.autoStart; }

    public void setAutoStart(boolean autoStart) { this.autoStart = autoStart; }

    public long getUsedBytes() { return this.bytesUsed; }

    private void setUsedBytes(long bytesUsed) { this.bytesUsed = bytesUsed; }

    public long getCpuAvailableBytes() { return this.cpuBytesAvail; }

    private void setCpuAvailableBytes(long cpuBytesAvail) { this.cpuBytesAvail = cpuBytesAvail; }

    public int getCpuCoProcessors() { return this.cpuCoProcessors; }

    private void setCpuCoProcessors(int cpuCoProcessors) { this.cpuCoProcessors = cpuCoProcessors; }

    public int getSelectedCpu() { return this.selectedCpu; }

    private void setSelectedCpu(int selectedCpu) { this.selectedCpu = selectedCpu; }

    public String getName() { return this.myName; }

    private void setName(@Nonnull String myName) { this.myName = myName; }

    public boolean hasNoCPU() { return this.noCPU; }

    private void setNoCPU(boolean noCPU) { this.noCPU = noCPU; }

    public boolean isSimulation() { return this.simulation; }

    private void setSimulation(boolean simulation) { this.simulation = simulation; }

    private Future<ICraftingJob> getJob() { return this.job; }

    public void setJob(Future<ICraftingJob> job) { this.job = job; }
}
